/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.manager;

import java.util.ArrayList;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dbaccess.DataConnectionManager;
import it.eng.spago.dbaccess.Utils;
import it.eng.spago.dbaccess.sql.DataConnection;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;

/**
 * @author Daniela Butano
 *
 * This class implements all methods (defined in PluginManagerIFace)
 * to manage plugins
 *
 */
public class PluginManager implements PluginManagerIFace {
     /**
    * This method provide to return a SourceBean representing all plugins
    * This method throws an exception if something goes wrong
    * @return a SourceBean of all event
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getAllPlugins() throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "PluginManager::getAllPlugins:invocato");
        try {
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ, "GET_ALL_PLUGINS", null);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "PluginManager::getAllPlugins:", ex);
            throw ex;
        }
    }

    /**
    * This method provide to return a SourceBean representing plugins with type passed in input
    * This method throws an exception if something goes wrong
    *
    * @return SourceBean of  plugins
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getPluginsForType(String typePlugin) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "PluginManager::getPluginsForType:invocato");
        try {
            ArrayList inputParameters = new ArrayList();
            inputParameters.add(typePlugin);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_PLUGINS_FOR_TYPE", inputParameters);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "PluginManager::getPluginsForType:", ex);
            throw ex;
        }
    }

     /**
    * This method provide to return all plugin type
    * This method throws an exception if something goes wrong
    *
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getPluginType() throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "PluginManager::getPluginType:invocato");
        try {
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ, "GET_PLUGIN_TYPE", null);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "PluginManager::getPluginType:", ex);
            throw ex;
        }
    }

    /**
    * This method provide to create a New plugin
    * This method throws an exception if something goes wrong
    *
    * @param String key, String description, String type of plugin to insert
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void createNewPlugin(String key, String description, String type) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "PluginManager::createNewPlugin: invocato");
        if (key != null && description != null && type != null) {
            try {
                ArrayList inputParameters = new ArrayList();
                inputParameters.add(key);
                inputParameters.add(description);
                inputParameters.add(type);
                QueryExecutor.executeProfilingQuery(QueryExecutor.CREATE,
                    "STORE_PLUGIN", inputParameters);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "PluginManager::createNewPlugin:", ex);
                throw ex;
            }
        } // if( key != null && description != null && type != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "PluginManager::createNewPlugin: key o description o type sono null");
    }

    /**
    * This method provide to remove the plugin identified by key, all associations
    * with events and all trigger associated to it
    * This method throws an exception if something goes wrong
    *
    * @param String key of plugin to remove
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void removePlugin(String key) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "PluginManager::removePlugin: invocato");
        if (key != null) {
            DataConnectionManager dataConnectionManager = null;
            DataConnection dataConnection = null;
            ConfigSingleton config = ConfigSingleton.getInstance();
            ArrayList inputParameters = new ArrayList();
            String query = null;
            try {
                dataConnectionManager = DataConnectionManager.getInstance();
                String pool = (String)config.getAttribute("events_pool");
                dataConnection = dataConnectionManager.getConnection(pool);
                dataConnection.initTransaction();
                //rimuovo tutti i trigger associati al plugin
                query = "REMOVE_TRIGGER_WITH_PLUGIN";
                inputParameters.add(key);
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
                //rimuovo tutte le associazioni con gli eventi	  			
                query = "REMOVE_PLUGIN_FROM_EVENT_WITH_PLUGIN";
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
                //rimuovo il plugin
                query = "REMOVE_PLUGIN";
                QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE, query, inputParameters);
                dataConnection.commitTransaction();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "EventManager::removePlugin:", ex);
                dataConnection.rollBackTransaction();
                throw ex;
            } // catch (Exception ex) try
            finally {
                Utils.releaseResources(dataConnection, null, null);
            } // finally try
        } // if (key != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "PluginManager::removePlugin: key vale null");
    }

    /**
    * This method provide to create a New plugin type
    * This method throws an exception if something goes wrong
    *
    * @param String key, String description of plugin type to insert
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void createNewPluginType(String key, String description) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "PluginManager::createNewPluginType: invocato");
        if (key != null && description != null) {
            try {
                ArrayList inputParameters = new ArrayList();
                inputParameters.add(key);
                inputParameters.add(description);
                QueryExecutor.executeProfilingQuery(QueryExecutor.CREATE,
                    "STORE_PLUGIN_TYPE", inputParameters);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                    "PluginManager::createNewPluginType:", ex);
                throw ex;
            }
        } // if( key != null && description != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "PluginManager::createNewPluginType: key o description sono null");
    }

    /**
    * This method provide to remove the plugin type
    * This method throws an exception if something goes wrong or if there are
    * some plugins associated
    *
    * @param String key of plugin type to remove
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void removePluginType(String key) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "PluginManager::removePluginType: invocato");
        if (key != null) {
            DataConnectionManager dataConnectionManager = null;
            DataConnection dataConnection = null;;
            SourceBean resultBean = null;
            ConfigSingleton config = ConfigSingleton.getInstance();
            ArrayList inputParameters = new ArrayList();
            String query = null;
            try {
                dataConnectionManager = DataConnectionManager.getInstance();
                String pool = (String)config.getAttribute("events_pool");
                dataConnection = dataConnectionManager.getConnection(pool);
                dataConnection.initTransaction();
                //verifico che non ci siano plugin associati,
                query = "GET_PLUGINS_FOR_TYPE_LOCK";
                inputParameters = new ArrayList();
                inputParameters.add(key);
                resultBean = QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.READ, query, inputParameters);
                //se non ci sono plugin associati viene rimosso il tipo plugin
                if (resultBean.getAttributeAsList("ROW").size() == 0) {
                    query = "REMOVE_PLUGIN_TYPE";
                    resultBean = QueryExecutor.executeProfilingQuery(dataConnection, QueryExecutor.DELETE,
                        query, inputParameters);
                } // if (resultBean.getAttributeAsVector("ROW").size() == 0)
                else
                    throw(new EMFUserError(EMFErrorSeverity.ERROR, 10102));
                dataConnection.commitTransaction();
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "PluginManager::removePluginType:", ex);
                dataConnection.rollBackTransaction();
                throw ex;
            } // catch (Exception ex) try
            finally {
                Utils.releaseResources(dataConnection, null, null);
            } // finally try
        } // if ( key != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "PluginManager::removePluginType: key vale null");
    }
}
