/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.event.manager;

import java.util.ArrayList;
import java.util.List;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.QueryExecutor;

/**
 * @author Daniela Butano
 *
 * This class implements all methods (defined in TriggerManagerIFace)
 * to manage triggers
 *
 */
public class TriggerManager implements TriggerManagerIFace {
    protected EventsFacadeIFace _eventsFacade = null;

	/**
	 * Constructor for TriggerHandler.
	 */
    public TriggerManager(EventsFacadeIFace eventsFacade) {
        _eventsFacade = eventsFacade;
    }

     /**
    * This method provide to return a SourceBean representing all triggers
    * This method throws an exception if something goes wrong
    * @return SourceBean representing all triggers
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getAllTriggers() throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::getAllTriggers:invocato");
        try {
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ, "GET_ALL_TRIGGERS", null);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "TriggerManager::getAllTriggers:", ex);
            throw ex;
        }
    }

     /**
    * This method provide to return a SourceBean representing all triggers related with the event identified by eventName
    * This method throws an exception if something goes wrong
    * @return a SourceBean representing triggers
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getAllTriggersByEvent(String eventName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::getAllTriggersByEvent:invocato");
        return getAllTriggersByEventPlugin(eventName, "");
    }

     /**
    * This method provide to return a SourceBean representing all triggers related with the plugin identified by pluginName
    * This method throws an exception if something goes wrong
    * @return a SourceBean representing triggers
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getAllTriggersByPlugin(String pluginName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::getAllTriggersByPlugin:invocato");
        return getAllTriggersByEventPlugin("", pluginName);
    }

     /**
    * This method provide to return a SourceBean representing all triggers related with the event
    * and plugin identified by eventName and pluginName
    * If eventName or pluginName are equals to "" the condition is ignored
    * This method throws an exception if something goes wrong
    * @return a SourceBean representing triggers
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getAllTriggersByEventPlugin(String eventName, String pluginName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
            "TriggerManager::getAllTriggersByEventPlugin:invocato");
        try {
            ArrayList parameters = new ArrayList();
            if (eventName.equals(""))
                eventName = "%";
            parameters.add(eventName);
            if (pluginName.equals(""))
                pluginName = "%";
            parameters.add(pluginName);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_ALL_TRIGGERS_BY_EVENT_PLUGIN", parameters);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "TriggerManager::getAllTriggersByEventPlugin:", ex);
            throw ex;
        }
    }

    /**
    * This method provide to return all triggers matching with attribute passed in input
    * If some atrribute is equals to "", the condition is ignored
    * This method throws an exception if something goes wrong
    * @return SourceBean representing all triggers matching
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public SourceBean getAllTriggers(String event, String plugin, String idProfile, String typeProfile) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::getAllTriggers: invocato");
        try {
            ArrayList parameters = new ArrayList();
            if (event.equals(""))
                event = "%";
            parameters.add(event);
            if (plugin.equals(""))
                plugin = "%";
            parameters.add(plugin);
            if (idProfile.equals(""))
                idProfile = "%";
            parameters.add(idProfile);
            if (typeProfile.equals(""))
                typeProfile = "%";
            parameters.add(typeProfile);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_ALL_TRIGGERS_BY_EVENT_PLUGIN_PROFILE", parameters);
            return resultBean;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "TriggerManager::getAllTriggers:", ex);
            throw ex;
        }
    }

     /**
    * This method provide to return a  list of pluginName which are related
    * with the event identified by eventName in some triggers
    * This method throws an exception if something goes wrong
    * @return a List of plugins
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public ArrayList getAllPluginsByEvent(String eventName) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::getAllPluginsByEvent: invocato");
        ArrayList allPluginByEvent = new ArrayList();
        try {
            ArrayList parameters = new ArrayList();
            parameters.add(eventName);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_ALL_PLUGIN_BY_EVENT", parameters);
            List rows = resultBean.getAttributeAsList("ROW");
            SourceBean row = null;
            int rowsNumber = rows.size();
            for (int i = 0; i < rowsNumber; i++) {
                row = (SourceBean)rows.get(i);
                allPluginByEvent.add((String)row.getAttribute(EventsConstants.TRIGGER_PLUGIN));
            }
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "TriggerManager::getAllPluginsByEvent:", ex);
        }
        return allPluginByEvent;
    }

     /**
    * This method provide to return true if exists some relationship between
    * the user identify by user and the event identified by eventName and the plugin or
    * if exists some relationship between a user's roles
    * the event identified by eventName and the plugin
    * This method throws an exception if something goes wrong
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public boolean isUserTriggered(String userId, String eventName, String pluginName) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::isUserTriggered: invocato");
        try {
            //....if exists some relationship between user, event, plugin
            ArrayList parameters = new ArrayList();
            parameters.add(userId);
            parameters.add(eventName);
            parameters.add(pluginName);
            SourceBean resultBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ, "IS_USER_TRIGGERED", parameters);
            List rows = resultBean.getAttributeAsList("ROW");
            int rowsNumber = rows.size();
            if (rowsNumber == 1)
                return true;
            //....or if exists some relationship between user's roles, event, plugin
            _eventsFacade.getAllRoles(userId);
            parameters = new ArrayList(2);
            parameters.add(eventName);
            parameters.add(pluginName);
            SourceBean triggersBean = QueryExecutor.executeProfilingQuery(QueryExecutor.READ,
                "GET_ALL_TRIGGERS_BY_EVENT_PLUGIN", parameters);
            List triggers = triggersBean.getAttributeAsList("ROW");
            SourceBean triggerBean = null;
            for (int i = 0; i < triggers.size(); i++) {
                triggerBean = (SourceBean)triggers.get(i);
                if (rows.contains(triggerBean.getAttribute(EventsConstants.TRIGGER_EVENT)))
                    return true;
            }
            return false;
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "TriggerManager::getAllTriggers:", ex);
        }
        return false;
    }

     /**
    * This method provide to return all user related with the event identified by eventName and the plugin
    * This method throws an exception if something goes wrong
    * @return a List of user
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public ArrayList getAllUsersTriggered(String eventName, String pluginName) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::getAllUsersTriggered: invocato");
        ArrayList usersTriggered = new ArrayList();
        //retrive all triggers related whit the eventName and pluginName passed in input
        List triggersByEventPlugin = getAllTriggersByEventPlugin(eventName, pluginName).getAttributeAsList("ROW");
        //for every trigger, retrive user,if type_profile = USER,
        //all user associated whit a role, if type_profile = ROLE,
        SourceBean triggerBean = null;
        String typeProfile = null;
        String userId = null;
        String roleId = null;
        for (int numRow = 0; numRow < triggersByEventPlugin.size(); numRow++) {
            triggerBean = (SourceBean)triggersByEventPlugin.get(numRow);
            typeProfile = (String)triggerBean.getAttribute(EventsConstants.TRIGGER_TYPE_PROFILE);
            if (typeProfile.equals(EventsConstants.USER_TYPE_PROFILE)) {
                userId = (String)triggerBean.getAttribute(EventsConstants.TRIGGER_ID_PROFILE);
                usersTriggered.add(userId);
            }
            else if (typeProfile.equals(EventsConstants.ROLE_TYPE_PROFILE)) {
                roleId = (String)triggerBean.getAttribute(EventsConstants.TRIGGER_ID_PROFILE);
                usersTriggered.addAll(_eventsFacade.getAllUsers(roleId));
            }
            else {
                //usersTriggered.addAll(_eventsFacade.getAllUsers());
            }
        }
        return usersTriggered;
    }

    /**
    * This method provide to remove the trigger identified by event, plugin, idProfile, typeProfile
    * This method throws an exception if something goes wrong
    *
    * @param event, plugin, idProfile, typeProfile of trigger to remove
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void removeTrigger(String event, String plugin, String idProfile, String typeProfile) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::removeTrigger: invocato");
        if (event != null && plugin != null && idProfile != null && typeProfile != null) {
            try {
                ArrayList inputParameters = new ArrayList();
                inputParameters.add(event);
                inputParameters.add(plugin);
                inputParameters.add(idProfile);
                inputParameters.add(typeProfile);
                QueryExecutor.executeProfilingQuery(QueryExecutor.DELETE,
                    "REMOVE_TRIGGER", inputParameters);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "TriggerManager::removeTrigger:", ex);
                throw ex;
            } // catch (Exception ex) try
        } // if (event != null && plugin != null && idProfile != null &&
        // typeProfile != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "TriggerManager::removeTrigger: plugin o event o idProfile o typeprofile sono null");
    }

    /**
    * This method provide to create a new trigger identified by event, plugin, idProfile, typeProfile
    * This method throws an exception if something goes wrong
    *
    * @param event, plugin, idProfile, typeProfile of trigger to create
    * @throws <{Exception}>  - if some problems occurs during operation
    */
    public void createNewTrigger(String event, String plugin, String idProfile, String typeProfile) throws Exception {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "TriggerManager::createNewTrigger: invocato");
        if (event != null && plugin != null && idProfile != null && typeProfile != null) {
            try {
                ArrayList inputParameters = new ArrayList();
                inputParameters.add(event);
                inputParameters.add(plugin);
                inputParameters.add(idProfile);
                inputParameters.add(typeProfile);
                QueryExecutor.executeProfilingQuery(QueryExecutor.CREATE,
                    "STORE_TRIGGER", inputParameters);
            } // try
            catch (Exception ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL, "TriggerManager::createNewTrigger:", ex);
                throw ex;
            } // catch (Exception ex) try
        } // if (event != null && plugin != null && idProfile != null &&
        // typeProfile != null)
        else
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                "TriggerManager::createNewTrigger: plugin o event o idProfile o typeprofile sono null");
    }
}
