/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.transcoding;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringBufferInputStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;

import org.apache.xalan.trace.PrintTraceListener;

import it.eng.spago.base.Constants;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;

/**
 * Questa classe effettua la transcodifica: data una stringa contente XML,
 * applica un XSL(gi processato, ovvero un'istanza di Templates) e produce
 * HTML, WML etc
 * Esempio di utilizzo della classe:
 *       Transcoder transcoder = new Transcoder();
 *       //out pu essere un'istanza di PrintWriter o OutputStream(vedere doc Xalan)
 *       Result result         = new StreamResult(out);
 *       transcoder.setXSL(template);
 *       transcoder.perform(list.perform().toXML(), result);
 *
 * @author Daniela Butano
 * @version 1.0
 */
public class Transcoder {
    private static final String TRACE_FILE_NAME = "transcoder";
    private static final String TRACE_FILE_PATH = "/tmp/";
    private static boolean _traceFileOpened = false;
    private static boolean _debug = false;
    private static FileWriter _fileWriter = null;
    private static PrintWriter _printWriter = null;
    private static PrintTraceListener _printTraceListener = null;

    private Transcoder() {
        super();
    } // private Transcoder()

    public static void perform(String xml, Templates xslTemplate, Result out) {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Transcoder::perform: xml\n" + xml);
        Source xmlSource = new StreamSource(new StringBufferInputStream(xml));
        try {
            Transformer transformer = xslTemplate.newTransformer();
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                "Transcoder::perform: transformer [" + transformer + "]");

            /*
                        if (transformer instanceof TransformerImpl) {
                            if (!_traceFileOpened) {
                                synchronized(Transcoder.class) {
                                    if (!_traceFileOpened)
                                        openTraceFile();
                                } // synchronized (Transcoder.class)
                            } // if (!_traceFileOpened)
                            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                                "Transcoder::perform: _debug [" + _debug + "]");
                            if (_debug) {
                                TransformerImpl transformerImpl = (TransformerImpl)transformer;
                                TraceManager traceManager = transformerImpl.getTraceManager();
                                traceManager.addTraceListener(_printTraceListener);
                                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG,
                                    "Transcoder::perform: trace listener aggiunto");
                                synchronized(Transcoder.class) {
                                    transformer.transform(xmlSource, out);
                                } // synchronized (Transcoder.class)
                            } // if (_debug)
                            else
                                transformer.transform(xmlSource, out);
                        } // if (transformer instanceof TransformerImpl)
                        else
            */

            transformer.transform(xmlSource, out);
        } // try
        catch (Exception ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "Transcoder::perform: ", ex);
        } // catch (Exception ex)
    } // public static void perform(String xml, Templates xslTemplate, Result out)

    private static void openTraceFile() {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Transcoder::openTraceFile: invocato");
        ConfigSingleton configure = ConfigSingleton.getInstance();
        String traceFilePath = (String)configure.getAttribute("TRACING.TRACE_PATH");
        if (traceFilePath == null)
            traceFilePath = TRACE_FILE_PATH;
        SimpleDateFormat formatter = new SimpleDateFormat("yMMdd");
        String dateString = formatter.format(new Date());
        String traceFileName = TRACE_FILE_NAME;
        String debug = (String)configure.getAttribute("TRACING.DEBUG");
        _debug = ((debug != null) && debug.equalsIgnoreCase("TRUE"));
        String fileName = "";
        if (ConfigSingleton.getRootPath() != null)
            fileName += ConfigSingleton.getRootPath();
        fileName += traceFilePath + traceFileName + dateString + ".log";
        try {
            _fileWriter = new FileWriter(fileName);
            _printWriter = new PrintWriter(_fileWriter, true);
            _printTraceListener = new PrintTraceListener(_printWriter);
            _printTraceListener.m_traceElements = true;
            _printTraceListener.m_traceGeneration = true;
            _printTraceListener.m_traceSelection = true;
            _printTraceListener.m_traceTemplates = true;
        } // try
        catch (IOException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "Transcoder::openTraceFile: ", ex);
            _debug = false;
            _fileWriter = null;
            _printWriter = null;
            _printTraceListener = null;
        } // catch (IOException ex)
        _traceFileOpened = true;
    } // private static void openTraceFile()

    private static void closeTraceFile() {
        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "Transcoder::closeTraceFile: invocato");
        _debug = false;
        _printTraceListener = null;
        try {
            _printWriter.close();
            _fileWriter.close();
        } // try
        catch (IOException ex) {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.MAJOR, "Transcoder::closeTraceFile: ", ex);
        } // catch (IOException ex)
        _printWriter = null;
        _fileWriter = null;
        _traceFileOpened = false;
    } // private static void closeTraceFile()
} // public class Transcoder
