/**

    Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.impl.SyntaxChecker;

import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.validator.GenericValidator;
/**
 * @author zoppello
 *
 * This class checks if a field is a valid number.
 * 
 * This implementation need that this validator should be configured with parameters regarding language
 * and country, with this information the notion of "decimal separator" will depend on Locale object
 * istantiate using this information
 * 
 * For Example:
 * 
 * <FIELD-VALIDATOR fieldType="NUMERIC" fieldValidatorClass="it.eng.spago.validation.fieldvalidators.NuemricValidator">
 *		<CONFIG>
 *			<LANGUAGE language="it"/>
 *			<COUNTRY country="IT"/>
 *		</CONFIG>
 *	</FIELD-VALIDATOR>
 *
 *	If the config section the Italian Locale will be used
 *
 *  This Validator will convert the original value in a java.lang.Double object if it succeed and  
 *  this will be putted  in TYPED_SERVICE_REQUEST with an alias if this is configured in validation.xml file
 *  
 */
public class NumericValidator extends AbstractFieldValidator {

	public static final int ERRORE_CAMPO_NUMERICO = 10106; 		
	/** 
	 * Implements the logic described in JavaDoc of the class
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#validateField(it.eng.spago.base.SourceBean, java.lang.String, java.lang.String, java.util.Map)
	 */
	public void validateField(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams) throws EMFValidationError {
		SourceBean config = getConfig();
		
		String language = "it";
		String country = "IT";
		
		if (config != null){
			language = (String)config.getAttribute("LANGUAGE.language");
			country = (String)config.getAttribute("COUNTRY.country");
		}
		
		Locale locale = new Locale(language, country);
		
		
		DecimalFormatSymbols decimalFormatSymbols = new DecimalFormatSymbols(locale);
		
		char decimalSeparator = decimalFormatSymbols.getDecimalSeparator();
		
		String regexp = "^[-+]?\\d*\\"+decimalSeparator+"?\\d*$";
		
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "NumericValidator::validateField");
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "NumericValidator::validateField: Validate ["+value+"] with regexp ["+ regexp + "]");
		
		
		List params = new ArrayList();
		params.add(fieldName);
		if (!GenericValidator.matchRegexp(value, regexp)){
			throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_CAMPO_NUMERICO,params);
		}
		
		//This Field Validator Class doesn't apply type conversion it apply only alias on field 
		String fieldAlias = (String)fieldValidationParams.get(SyntaxChecker.PARAM_FIELD_ALIAS);
		if (fieldAlias == null){
			fieldAlias = fieldName;
		}
		
		
		if (value.startsWith("+")){
			value = value.substring(1);
		}
		
		Double convertedValue  = null;
		try {
	        Number number = NumberFormat.getNumberInstance(locale).parse(value);
	        convertedValue = new Double(number.doubleValue());
	        saveTypedValue(serviceRequest, fieldAlias, convertedValue);
	    } catch (ParseException e) {
	    	throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,ERRORE_CAMPO_NUMERICO,params);
	    }
	}
	
	
}
